"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArgsParser = exports.ParsedArgumentsCollection = void 0;
var exceptions_1 = require("./exceptions");
var utils_1 = require("./utils");
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
var ParsedArgumentsCollection = /** @class */ (function () {
    function ParsedArgumentsCollection() {
        this.args = {};
    }
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    ParsedArgumentsCollection.prototype.add = function (name, value) {
        this.args[this.formatName(name)] = value;
        return this;
    };
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    ParsedArgumentsCollection.prototype.get = function (name) {
        var _a;
        return ((_a = this.args[this.formatName(name)]) !== null && _a !== void 0 ? _a : undefined);
    };
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    ParsedArgumentsCollection.prototype.has = function (name) {
        return this.args[this.formatName(name)] !== undefined;
    };
    Object.defineProperty(ParsedArgumentsCollection.prototype, "all", {
        /**
         * Returns all arguments as a record.
         */
        get: function () {
            return __assign({}, this.args);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    ParsedArgumentsCollection.prototype.formatName = function (name) {
        return name.replace(/^--/, '');
    };
    return ParsedArgumentsCollection;
}());
exports.ParsedArgumentsCollection = ParsedArgumentsCollection;
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
var ArgsParser = /** @class */ (function () {
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    function ArgsParser(args) {
        var e_1, _a;
        if (args === void 0) { args = []; }
        this.argsMap = new Map();
        this.aliasMap = new Map();
        this.usedArgs = new Set();
        try {
            for (var args_1 = __values(args), args_1_1 = args_1.next(); !args_1_1.done; args_1_1 = args_1.next()) {
                var arg = args_1_1.value;
                this.addArgument(arg);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (args_1_1 && !args_1_1.done && (_a = args_1.return)) _a.call(args_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    ArgsParser.prototype.addArgument = function (config) {
        this.checkArgumentConfig(config);
        this.argsMap.set(config.name, this.formatArgConfig(config));
        this.usedArgs.add(config.name);
        if (config.alias !== undefined) {
            this.aliasMap.set(config.alias, config.name);
            this.usedArgs.add(config.alias);
        }
        return this;
    };
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argsString - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     * @throws ArgumentValueError if a required argument is missing or an argument value is invalid.
     */
    ArgsParser.prototype.parse = function (argsString) {
        var e_2, _a, e_3, _b;
        var required = new Set(__spreadArray([], __read(this.argsMap.values()), false).filter(function (x) { return x.required; }));
        var notEmpty = new Set(__spreadArray([], __read(this.argsMap.values()), false).filter(function (x) { return x.notEmpty; }));
        var hasDefault = new Set(__spreadArray([], __read(this.argsMap.values()), false).filter(function (x) { return x.default !== undefined; }));
        var parsedArgs = (0, utils_1.parseArgsString)(argsString);
        var result = new ParsedArgumentsCollection();
        try {
            for (var _c = __values(Object.entries(parsedArgs)), _d = _c.next(); !_d.done; _d = _c.next()) {
                var _e = __read(_d.value, 2), key = _e[0], value = _e[1];
                var argConfig = this.getArgConfig(key);
                if (notEmpty.has(argConfig) && value === '') {
                    throw new exceptions_1.ArgumentValueError("Argument ".concat(this.formatNameWithAlias(argConfig), " value cannot be empty."));
                }
                if (argConfig.allowedValues !== undefined && !argConfig.allowedValues.includes(value)) {
                    throw new exceptions_1.ArgumentValueError("Argument ".concat(this.formatNameWithAlias(argConfig), " value must be one of ").concat(argConfig.allowedValues.join(', '), "."));
                }
                if (argConfig.validator !== undefined && !argConfig.validator(value)) {
                    throw new exceptions_1.ArgumentValueError("Argument ".concat(this.formatNameWithAlias(argConfig), " value is invalid."));
                }
                required.delete(argConfig);
                hasDefault.delete(argConfig);
                result.add(argConfig.name, (0, utils_1.castArgValue)(value, argConfig.type, argConfig.multiple, argConfig.default));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_2) throw e_2.error; }
        }
        try {
            for (var hasDefault_1 = __values(hasDefault), hasDefault_1_1 = hasDefault_1.next(); !hasDefault_1_1.done; hasDefault_1_1 = hasDefault_1.next()) {
                var argConfig = hasDefault_1_1.value;
                result.add(argConfig.name, argConfig.default);
                required.delete(argConfig);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (hasDefault_1_1 && !hasDefault_1_1.done && (_b = hasDefault_1.return)) _b.call(hasDefault_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
        if (required.size > 0) {
            throw new exceptions_1.ArgumentValueError("Required arguments not provided: ".concat(__spreadArray([], __read(required), false).map(function (x) { return x.name; }).join(', ')));
        }
        return result;
    };
    /**
     * Formats an argument configuration.
     * @param config - The argument configuration.
     * @returns A formatted argument configuration.
     */
    ArgsParser.prototype.formatArgConfig = function (config) {
        var _a, _b, _c;
        return __assign(__assign({}, config), { required: (_a = config.required) !== null && _a !== void 0 ? _a : false, notEmpty: (_b = config.notEmpty) !== null && _b !== void 0 ? _b : false, multiple: (_c = config.multiple) !== null && _c !== void 0 ? _c : false });
    };
    /**
     * Formats the argument name and alias for error messages.
     * @param argConfig - The argument configuration.
     * @returns A formatted string with the argument name and alias.
     */
    ArgsParser.prototype.formatNameWithAlias = function (argConfig) {
        return "".concat(argConfig.name, " ").concat(argConfig.alias ? "(".concat(argConfig.alias, ")") : '');
    };
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    ArgsParser.prototype.checkArgumentConfig = function (config) {
        if (!config.name.startsWith('--')) {
            throw new exceptions_1.AddArgumentError("Argument name must start with '--' (\"".concat(config.name, "\" given)."));
        }
        if (config.alias !== undefined && !config.alias.startsWith('-')) {
            throw new exceptions_1.AddArgumentError("Argument alias must start with '-' (\"".concat(config.alias, "\" given)."));
        }
        if (this.usedArgs.has(config.name)) {
            throw new exceptions_1.AddArgumentError("Argument ".concat(config.name, " already exists."));
        }
        if (config.alias !== undefined && this.usedArgs.has(config.alias)) {
            throw new exceptions_1.AddArgumentError("Argument ".concat(config.alias, " already exists."));
        }
    };
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding FormattedArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    ArgsParser.prototype.getArgConfig = function (key) {
        var _a;
        var config = this.argsMap.get((_a = this.aliasMap.get(key)) !== null && _a !== void 0 ? _a : key);
        if (config === undefined) {
            throw new exceptions_1.ArgumentNameError("Unknown argument ".concat(key, "."));
        }
        return config;
    };
    return ArgsParser;
}());
exports.ArgsParser = ArgsParser;
//# sourceMappingURL=classes.js.map